@@ -25,7 +25,7 @@ extern "C" {
 
 #define RATE_GRANULARITY	100000000	 /* nsec between -L rate chunks */
 #define RATE_BURST_WINDOW	5	 	 /* rate burst window (multiples of rate) */
-#define REMOTE_INTERVAL		100000000	 /* nsec between checks for -R */
+#define REMOTE_INTERVAL		100000000	 /* nsec between checks for -R and -Q */
 #define BUFFER_SIZE		(size_t) 409600	 /* default transfer buffer size */
 #define BUFFER_SIZE_MAX		(size_t) 524288	 /* max auto transfer buffer size */
 #define MAX_READ_AT_ONCE	(size_t) 524288	 /* max to read() in one go */
@@ -131,6 +131,12 @@ typedef uint16_t pvdisplay_bytecount_t;
 typedef uint16_t pvdisplay_width_t;
 #define PVDISPLAY_WIDTH_MAX (65535)	/* UINT16_MAX */
 
+/*
+ * Structure defining the current state of a single watched file descriptor.
+ */
+struct pvwatchfd_s;
+typedef /*@null@*/ struct pvwatchfd_s *pvwatchfd_t;
+
 /* String pointer, that is the only pointer to this resource, that can be null. */
 typedef /*@only@*/ /*@null@*/ char * nullable_string_t;
 
@@ -163,9 +169,15 @@ struct pvstate_s {
 	/*********************************
 	 * Items to watch with --watchfd *
 	 *********************************/
-	struct pvwatchspec_s {
-		/*@only@*/ /*@null@*/ pid_t *pid; /* array of processes to watch fds of */
-		/*@only@*/ /*@null@*/ int *fd;	/* array of fds to watch in each one (0=all) */
+	struct {
+		/*@only@*/ /*@null@*/
+		struct pvwatcheditem_s {	/* array of PID or PID:FD items */
+			pid_t pid;		 /* watched PID */
+			int fd;			 /* watched fd, or -1 for all */
+			pvwatchfd_t info_array;	 /* watch information for each fd */
+			int array_length;	 /* length of watch info array */
+			bool finished;		 /* "PID:FD": fd closed; or PID gone */
+		} *watching;
 		unsigned int count;	/* number of items in these arrays */
 		bool multiple_pids;	/* true if more than one distinct PID */
 	} watchfd;
@@ -179,7 +191,8 @@ struct pvstate_s {
 		double delay_start;              /* delay before first display */
 		/*@only@*/ /*@null@*/ char *name;		 /* display name */
 		/*@only@*/ /*@null@*/ char *format_string;	 /* output format string */
-		/*@only@*/ /*@null@*/ char *extra_format_string; /* extra format string */
+		/*@only@*/ /*@null@*/ char *extra_display_spec;  /* full spec for extra displays */
+		/*@only@*/ /*@null@*/ char *extra_format_string; /* extra format string alone */
 		/*@null@*/ char *output_name;    /* name of the output, for diagnostics */
 		/*@null@*/ char *default_bar_style; /* which bar style to use by default */
 		off_t error_skip_block;          /* skip block size, 0 for adaptive */
@@ -193,6 +206,17 @@ struct pvstate_s {
 		pvdisplay_width_t width;         /* screen width */
 		unsigned int height;             /* screen height */
 		unsigned int extra_displays;	 /* bitmask of extra display destinations */
+		struct {			 /* old-style format options (used by -R) */
+			size_t lastwritten;	  /* --last-written (amount) */
+			bool progress;		  /* --progress */
+			bool timer;		  /* --timer */
+			bool eta;		  /* --eta */
+			bool fineta;		  /* --fineta */
+			bool rate;		  /* --rate */
+			bool average_rate;	  /* --average-rate */
+			bool bytes;		  /* --bytes */
+			bool bufpercent;	  /* --buffer-percent */
+		} format_option;
 		bool force;                      /* display even if not on terminal */
 		bool cursor;                     /* use cursor positioning */
 		bool numeric;                    /* numeric output only */
@@ -230,13 +254,16 @@ struct pvstate_s {
 		struct sigaction old_sigterm;
 #ifdef PV_REMOTE_CONTROL
 		struct sigaction old_sigusr2;
+		struct sigaction old_sigusr1;
 #endif
 		struct sigaction old_sigalrm;
-		struct timespec tstp_time;	 /* see pv_sig_tstp() / __cont() */
-		struct timespec toffset;	 /* total time spent stopped */
+		struct timespec when_tstp_arrived;	 /* see pv_sig_tstp() / __cont() */
+		struct timespec total_stoppage_time;	 /* total time spent stopped */
 #ifdef PV_REMOTE_CONTROL
 		volatile sig_atomic_t rxusr2;	 /* whether SIGUSR2 was received */
-		volatile pid_t sender;		 /* PID of sending process for SIGUSR2 */
+		volatile pid_t sender_usr2;	 /* PID of sending process for SIGUSR2 */
+		volatile sig_atomic_t rxusr1;	 /* whether SIGUSR1 was received */
+		volatile pid_t sender_usr1;	 /* PID of sending process for SIGUSR1 */
 #endif
 	} signal;
 
@@ -373,9 +400,9 @@ struct pvstate_s {
 	 * The transfer buffer is used for moving data from the input files
 	 * to the output when splice() is not available.
 	 *
-	 * If buffer_size is smaller than pv__target_bufsize, then
+	 * If buffer_size is smaller than control.target_buffer_size, then
 	 * pv_transfer() will try to reallocate transfer_buffer to make
-	 * buffer_size equal to pv__target_bufsize.
+	 * buffer_size equal to control.target_buffer_size.
 	 *
 	 * Data from the input files is read into the buffer; read_position
 	 * is the offset in the buffer that we've read data up to.
@@ -453,6 +480,37 @@ typedef struct pvtransfercalc_s *pvtransfercalc_t;
 typedef struct pvcursorstate_s *pvcursorstate_t;
 typedef struct pvtransferstate_s *pvtransferstate_t;
 
+/*
+ * Structure defining the current state of a single watched file descriptor.
+ * The full definition needs to go here as it refers to sub-structures of
+ * the main state, defined above.
+ */
+struct pvwatchfd_s {
+	struct pvtransientflags_s flags;	/* transient flags */
+	struct pvtransferstate_s transfer;	/* transfer state */
+	struct pvtransfercalc_s calc;	 /* calculated transfer state */
+	struct pvdisplay_s display;	 /* display data */
+#ifdef __APPLE__
+#else
+	char file_fdinfo[PV_SIZEOF_FILE_FDINFO]; /* path to /proc fdinfo file */
+	char file_fd[PV_SIZEOF_FILE_FD];	 /* path to /proc fd symlink  */
+#endif
+	char file_fdpath[PV_SIZEOF_FILE_FDPATH]; /* path to file that was opened */
+	/*@keep@ */ char display_name[PV_SIZEOF_DISPLAY_NAME]; /* name to show on progress bar */
+	struct stat sb_fd;		 /* stat of fd symlink */
+	struct stat sb_fd_link;		 /* lstat of fd symlink */
+	off_t size;			 /* size of whole file, 0 if unknown */
+	off_t position;			 /* position last seen at */
+	struct timespec start_time;	 /* time we started watching the fd */
+	struct timespec end_time;	 /* time the fd was marked as closed */
+	struct timespec total_stoppage_time;	 /* total time spent stopped */
+	pid_t watch_pid;		 /* PID the fd belongs to */
+	int watch_fd;			 /* fd to watch */
+	bool closed;			 /* true once the fd is closed */
+	bool displayable;		 /* false if not displayable */
+	bool unused;			 /* true if free for re-use */
+};
+
 /*
  * Read-only counterparts to the above structure pointers, to be used in
  * function declarations where the function definitely shouldn't be altering
@@ -499,36 +557,6 @@ struct pvdisplay_component_s {
 	bool dynamic;			 /* whether it can scale with screen size */
 };
 
-
-/*
- * Structure defining the current state of a single watched file descriptor.
- */
-struct pvwatchfd_s {
-	struct pvtransientflags_s flags;	/* transient flags */
-	struct pvtransferstate_s transfer;	/* transfer state */
-	struct pvtransfercalc_s calc;	 /* calculated transfer state */
-	struct pvdisplay_s display;	 /* display data */
-#ifdef __APPLE__
-#else
-	char file_fdinfo[PV_SIZEOF_FILE_FDINFO]; /* path to /proc fdinfo file */
-	char file_fd[PV_SIZEOF_FILE_FD];	 /* path to /proc fd symlink  */
-#endif
-	char file_fdpath[PV_SIZEOF_FILE_FDPATH]; /* path to file that was opened */
-	/*@keep@ */ char display_name[PV_SIZEOF_DISPLAY_NAME]; /* name to show on progress bar */
-	struct stat sb_fd;		 /* stat of fd symlink */
-	struct stat sb_fd_link;		 /* lstat of fd symlink */
-	off_t size;			 /* size of whole file, 0 if unknown */
-	off_t position;			 /* position last seen at */
-	struct timespec start_time;	 /* time we started watching the fd */
-	struct timespec end_time;	 /* time the fd was marked as closed */
-	pid_t watch_pid;		 /* PID the fd belongs to */
-	int watch_fd;			 /* fd to watch */
-	bool closed;			 /* true once the fd is closed */
-	bool displayable;		 /* false if not displayable */
-	bool unused;			 /* true if free for re-use */
-};
-typedef struct pvwatchfd_s *pvwatchfd_t;
-
 void pv_error(char *, ...);
 
 int pv_main_loop(pvstate_t);
@@ -605,6 +633,7 @@ void pv_freecontents_display(pvdisplay_t);
 void pv_freecontents_transfer(pvtransferstate_t);
 void pv_freecontents_calc(pvtransfercalc_t);
 void pv_freecontents_watchfd(pvwatchfd_t);
+void pv_freecontents_watchfd_items(struct pvwatcheditem_s *, unsigned int);
 
 void pv_write_retry(int, const char *, size_t);
 void pv_tty_write(readonly_pvtransientflags_t, const char *, size_t);
@@ -620,10 +649,7 @@ void pv_sig_allowpause(void);
 void pv_sig_checkbg(void);
 void pv_sig_nopause(void);
 
-void pv_remote_init(pvstate_t);
 bool pv_remote_check(pvstate_t);
-void pv_remote_fini(pvstate_t);
-int pv_remote_set(pvstate_t);
 
 int pv_watchfd_info(pvstate_t, pvwatchfd_t, bool);
 bool pv_watchfd_changed(pvwatchfd_t);
