@@ -360,6 +360,7 @@ void display_help(void)
 		{ "-U", "--store-and-forward", N_("FILE"),
 		 N_("write all input to FILE before writing to output"),
 		 { 0, 0, 0, 0} },
+		{ "", NULL, NULL, NULL, { 0, 0, 0, 0} },
 		{ "-d", "--watchfd", N_("PID[:FD]|=NAME|@LISTFILE"),
 		 N_("watch file FD opened by process PID"),
 		 { 0, 0, 0, 0} },
@@ -367,6 +368,9 @@ void display_help(void)
 		{ "-R", "--remote", N_("PID"),
 		 N_("update settings of process PID"),
 		 { 0, 0, 0, 0} },
+		{ "-Q", "--query", N_("PID"),
+		 N_("show progress of process PID"),
+		 { 0, 0, 0, 0} },
 #endif				/* PV_REMOTE_CONTROL */
 		{ "", NULL, NULL, NULL, { 0, 0, 0, 0} },
 		{ "-P", "--pidfile", N_("FILE"),
