@@ -23,10 +23,6 @@
 #include <langinfo.h>
 #endif
 
-int pv_remote_set(opts_t, pvstate_t);
-void pv_remote_init(void);
-void pv_remote_fini(void);
-
 /*
  * Write a PID file, returning nonzero on error.  Write it atomically, such
  * that the file either exists and contains the PID, or is not updated at
@@ -338,23 +334,6 @@ int main(int argc, char **argv)
 		/*@+mustfreefresh@ */
 	}
 
-	/*
-	 * -R specified - send the message, then exit.
-	 */
-	if (opts->remote > 0) {
-		/* Initialise signal handling. */
-		pv_sig_init(state);
-		/* Send the message. */
-		retcode = pv_remote_set(opts, state);
-		/* Close down the signal handling. */
-		pv_sig_fini(state);
-		/* Free resources. */
-		pv_state_free(state);
-		opts_free(opts);
-		/* Early exit. */
-		return retcode;
-	}
-
 	/*
 	 * Write a PID file if -P was specified.
 	 */
@@ -490,6 +469,31 @@ int main(int argc, char **argv)
 		}
 	}
 
+	/* Initialise the signal handling. */
+	pv_sig_init(state);
+
+	/*
+	 * Get the total size, in query mode.
+	 *
+	 * Note that this uses signals, so signal handling has to have been
+	 * set up first.
+	 */
+	if (PV_ACTION_QUERY == opts->action) {
+		opts->size = 0;
+		retcode = pv_remote_transferstate_fetch(state, opts->query, &(opts->size), false);
+		if (0 != retcode) {
+			pv_sig_fini(state);
+			pv_state_free(state);
+			opts_free(opts);
+			return retcode;
+		}
+		/* As above - no ETA if the size is unknown. */
+		if (opts->size < 1) {
+			can_have_eta = false;
+			debug("%s", "size unknown - ETA disabled");
+		}
+	}
+
 	/*
 	 * Copy the remaining parameters from the options into the main
 	 * state.
@@ -532,29 +536,30 @@ int main(int argc, char **argv)
 	debug("%s: %s", "terminal_supports_utf8", terminal_supports_utf8 ? "true" : "false");
 	pv_state_set_terminal_supports_utf8(state, terminal_supports_utf8);
 
-	/* Initialise the signal handling. */
-	pv_sig_init(state);
-
 	/* Run the appropriate main loop. */
 	switch (opts->action) {
 	case PV_ACTION_NOTHING:
 		break;
 	case PV_ACTION_TRANSFER:
 		/* Normal "transfer data" mode. */
-		pv_remote_init();
 		retcode = pv_main_loop(state);
-		pv_remote_fini();
 		break;
 	case PV_ACTION_STORE_AND_FORWARD:
 		/* Store-and-forward transfer mode. */
-		pv_remote_init();
 		retcode = pv__store_and_forward(state, opts, can_have_eta);
-		pv_remote_fini();
 		break;
 	case PV_ACTION_WATCHFD:
 		/* "Watch file descriptor(s) of another process" mode. */
 		retcode = pv_watchfd_loop(state);
 		break;
+	case PV_ACTION_REMOTE_CONTROL:
+		/* Change the options of another running pv. */
+		retcode = pv_remote_set(state, opts->remote);
+		break;
+	case PV_ACTION_QUERY:
+		/* Query the progress of another running pv. */
+		retcode = pv_query_loop(state, opts->query);
+		break;
 	}
 
 	/* Clear up the PID file, if one was written. */
